import java.util.HashSet;

/**
 * Class for representing a binary image (as a 2D boolean array)
 */
class BinaryImage {
  public static final boolean WHITE = false;
  public static final boolean BLACK = true;

  /** 2D array representing the image pixels */
  final boolean[][] pixels;

  /**
   * Create an image of n rows and m columns
   * 
   * @param n
   * @param m
   */
  BinaryImage(int n, int m) {
    assert n >= 1 && m >= 1;
    this.pixels = new boolean[n][m];
  }

  /**
   * Set the pixel BLACK
   * 
   * @param i
   * @param j
   */
  void set(int i, int j) {
    if (i >= 0 && i < pixels.length && j >= 0 && j < pixels[0].length)
      this.pixels[i][j] = BLACK;
  }

  /**
   * Set the pixel WHITE
   * 
   * @param i
   * @param j
   */
  void clear(int i, int j) {
    if (i >= 0 && i < pixels.length && j >= 0 && j < pixels[0].length)
      this.pixels[i][j] = WHITE;
  }

  /**
   * Inverse the color of a pixel
   */
  public void inversePixel(int i, int j) {
    if (i >= 0 && i < pixels.length && j >= 0 && j < pixels[0].length)
      this.pixels[i][j] = !this.pixels[i][j];
  }

  @Override
  public int hashCode() {
    // throw new Error("Méthode int hashCode() à compléter (Question 1.1)");
    int h = 0;
    for (boolean[] row : this.pixels)
      for (boolean b : row)
        h = 31 * h + (b ? 1 : 0);
    return h;
  }

  /**
   * Check equality between two images (pixel by pixel)
   */
  @Override
  public boolean equals(Object o) {
    BinaryImage that = (BinaryImage) o;
    // throw new Error("Méthode boolean equals(Object o) à compléter (Question 1.1)");
    if (this.pixels.length != that.pixels.length)
      return false;
    for (int i = 0; i < this.pixels.length; i++) {
      boolean[] r1 = this.pixels[i], r2 = that.pixels[i];
      if (r1.length != r2.length)
        return false;
      for (int j = 0; j < r1.length; j++)
        if (r1[j] != r2[j])
          return false;
    }
    return true;
  }

  /**
   * Given a collection of binary images (stored in the input array), remove all
   * duplicates and return a new array storing the images (without duplicates)
   * 
   * @param a an array storing the input images
   * @return a new array storing all input images without repetitions
   */
  static BinaryImage[] deleteDuplicates(BinaryImage[] t) {
    // throw new Error("Méthode BinaryImage[] deleteDuplicates(BinaryImage[] a) à compléter (Question 1.2)");
    HashSet<BinaryImage> s = new HashSet<>();
    for (BinaryImage im : t)
      s.add(im);
    BinaryImage[] r = new BinaryImage[s.size()];
    int i = 0;
    for (BinaryImage im : s)
      r[i++] = im;
    return r;
  }

}
